//
//	BasicWinProc.cpp
//
//	This class provides a windows procedure (useful only in Win32 implementations) that
//	hooks up the framework's command-handling systems.  It processes default commands
//	in the default ways, and passes almost all other commands to the command-processor
//	for execution (effectively crippling Win32's automatic message routing, and reverting
//	it back to a Mac-like central event loop model).
//
//	Window-based events get routed to a PortWindow, if there is one, otherwise we just
//	call the default window procedure (or Dialog Procedure, if it's a dialog) with them.
#include "UniversalSymbols.h"

#if (UT_TARGET_WIN32)
#include "BasicWinProc.h"
#include "Controls.h"
#include "Window.h"

#include <ShellApi.h>

using namespace PortableViews;

LRESULT CommonWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
	{
	LRESULT returnVal = 1;
	PortWindow* avWin = PortWindow::PlatformWindowToPortWindow(hwnd);

	switch (iMsg)
		{
		case WM_CREATE :
			return 0;
			break;
						
		case WM_COMMAND : 
			if (lParam == NULL)
				{
				// Command is from a menu, probably.
				}
			else
				{
				// It's from a control (child window), handle it differently.
				WORD notificationCode = HIWORD(wParam);
				WORD controlID = LOWORD(wParam);
				
				// Check to see that it's an event we're interested in (Windows
				// sends tons of these around for all sorts of nitpicky little
				// things like highlight change and focus-grabbing, we only care
				// about things which activate or change the value of the control).
				bool isUseful = false;
				bool isLBDoubleClick = false;
				
				char buffer[255];
				GetClassName((HWND)lParam, buffer, 255);
				
				if (strcmp(buffer, "Edit") == 0 && notificationCode == EN_CHANGE)
					isUseful = true;
				if (strcmp(buffer, "Button") == 0 && notificationCode == BN_CLICKED)
					isUseful = true;
				if (strcmp(buffer, "ListBox") == 0 && notificationCode == LBN_SELCHANGE)
					isUseful = true;
				if (strcmp(buffer, "ComboBox") == 0 && notificationCode == CBN_SELCHANGE)
					isUseful = true;
				if (strcmp(buffer, "ListBox") == 0 && notificationCode == LBN_DBLCLK)
					isLBDoubleClick = true;

				if (isUseful && avWin != NULL)
					avWin->ControlValueChanged((platformControl)lParam, controlID);
					
				if (isLBDoubleClick)
					avWin->ListBoxDoubleClicked((platformControl)lParam, controlID);
				}
			return 0;
			break;


		case WM_PAINT :
				{
				// If we're not handling it, we still need to call BeginPaint/EndPaint,
				// or the message will remain undead until processed.
				HDC theDC;
				PAINTSTRUCT ps;
				theDC = BeginPaint(hwnd, &ps);
				EndPaint(hwnd, &ps);
				}
			return 0;
			break;
			
		case WM_SIZE:
			if (wParam == SIZE_MAXIMIZED || wParam == SIZE_RESTORED)
				{
				int width = LOWORD(lParam);
				int height = HIWORD(lParam);
				if (avWin != NULL)
					{
					RECT cSize;
					GetClientRect(hwnd, &cSize);
					avWin->PostResize(cSize.right-cSize.left, cSize.bottom-cSize.top);
					return 0;
					}
				}
			break;
			

		case WM_DESTROY :
			if (avWin != NULL)
				avWin->Close();
			if (hwnd == gAppWindow)
				{
				PostQuitMessage (0);
				}
			return 0;
			break;			
         }

	 return returnVal;
	}

LRESULT CALLBACK StandardWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
	{
	LRESULT retVal = CommonWndProc(hwnd, iMsg, wParam, lParam);
	if (retVal != 0)
	 	return DefWindowProc(hwnd, iMsg, wParam, lParam);
	 else 
	 	return 0;
    }
#endif