//@collection:Controls
//@descr
//	A few routines for creating, setting and getting the values of, and 
//  modifying controls.
//@endc

#ifndef 	AO_CONTROLS
#define		AO_CONTROLS

#include  "PortString.h"

namespace PortableViews {

typedef	sInt16	controlType;

const controlType kAOControlScrollbar = 20000;
const controlType kAOControlButton = 20001;
const controlType kAOControlCheckbox = 20002;
const controlType kAOControlRadio = 20003;
const controlType kAOControlListBox = 20004;
const controlType kAOControlPulldown = 20005;
const controlType kAOControlStaticText = 20006;
const controlType kAOControlEditText = 20007;
const controlType kAOControlIcon = 20008;
const controlType kAOControlPicture = 20009;
const controlType kAOControlFramedBox = 20010;


const sInt32	kAONoSelection = -1; // Returned when asking for a nonexistant list box/menu selection.	


platformControl	CreatePlatformControl(controlType type, platformWindow parent, 
				const PortString& label, platformRect* pos, 
				sInt16 commandId, sInt16 initValue, bool isDefault = false,
				sInt16 resourceRef = 0, const PortString* const initialText = NULL);

void	MovePlatformControl(platformControl theControl, platformRect* newLoc);
void	MoveScrollbarControl(platformControl theControl, platformRect* newLoc);

void 	SetScrollbarControlRange(platformControl theBar, sInt16 minVal, sInt16 maxVal);
void 	SetScrollbarControlValue(platformControl theBar, sInt16 theVal);

void	DisablePlatformControl(platformControl theControl);
void	EnablePlatformControl(platformControl theControl);

void	HidePlatformControl(platformControl theControl);
void	ShowPlatformControl(platformControl theControl);

sInt16	GetScrollbarControlValue(platformControl theBar);

sInt16	GetPlatformCheckOrRadioValue(platformControl theControl);
void	SetPlatformCheckOrRadioValue(platformControl theControl, sInt16 theValue);

// Couple options on Edit Text controls
void	SetEditTextText(platformControl theControl, const PortString& theText);
void	GetEditTextText(platformControl theControl, PortString& theText);

// Only one for static text
void	SetStaticTextText(platformControl theControl, const PortString& theText);

// Lots of options on List Boxes...
sInt16		CountListBoxItems(platformControl theList);
sInt16		GetListBoxSelection(platformControl theList);
PortString	GetTextOfListBoxItem(platformControl theList, sInt16 theIndex);
void		DeleteListBoxItem(platformControl theList, sInt16 theIndex);
void		AddListBoxItem(platformControl theList, sInt16 atPos, const PortString& theItem);
void		SelectListBoxItem(platformControl theList, sInt16 atPos);

// And popup menu controls...
sInt16		CountMenuControlItems(platformControl theMenu);
sInt16		GetMenuControlSelection(platformControl theMenu);
PortString	GetTextOfMenuControlItem(platformControl theMenu, sInt16 theIndex);
void		DeleteMenuControlItem(platformControl theMenu, sInt16 theIndex);
void		AddMenuControlItem(platformControl theMenu, sInt16 atPos, const PortString& theItem);
void		SelectMenuControlItem(platformControl theMenu, sInt16 atPos);


} // namespace PortableViews
#endif