//	A few routines for setting and getting the values of controls.
//
#include "UniversalSymbols.h"

#if	(UT_TARGET_WIN32)

#include "PortString.h"

#include "Controls.h"
#include <Windows.h>

namespace PortableViews {

// Window control styles
const long kBaseControlStyle =  WS_CHILD | WS_VISIBLE | WS_TABSTOP;
const long kButtonStyle = kBaseControlStyle | BS_PUSHBUTTON | ES_LEFT;
const long kRadioStyle = kBaseControlStyle | BS_RADIOBUTTON;
const long kScrollBarStyle = WS_CHILD | WS_VISIBLE | SBS_BOTTOMALIGN;
const long kCheckBoxStyle = kBaseControlStyle | BS_CHECKBOX;
const long kIconStyle = SS_ICON | WS_CHILD | WS_VISIBLE;
const long kPictureStyle = SS_BITMAP | WS_CHILD | WS_VISIBLE;
const long kStaticTextStyle = SS_LEFT | WS_CHILD | WS_VISIBLE;
const long kEditTextStyle =  kBaseControlStyle | ES_LEFT | ES_AUTOHSCROLL;
const long kListBoxStyle = kBaseControlStyle | LBS_NOTIFY | WS_VSCROLL | LBS_DISABLENOSCROLL | LBS_NOINTEGRALHEIGHT;
const long kPullDownStyle = kBaseControlStyle | CBS_DROPDOWNLIST | ES_LEFT;
const long kFrameStyle = kBaseControlStyle | BS_GROUPBOX;

platformControl	CreatePlatformControl(controlType type, platformWindow parent, 
		const PortString& label, platformRect* pos, sInt16 commandID, sInt16 initValue,
		bool isDefault, sInt16 resourceRef, const PortString* const initialText)
	{
	RECT newRect = *pos;
	
	platformControl ctrl;
	long style;
	switch (type)
		{
		case kAOControlScrollbar:
			style = kScrollBarStyle;
			if ((pos->right - pos->left) > (pos->bottom - pos->top))
				style |= SBS_HORZ;
			else
				style |= SBS_VERT;
			ctrl = CreateWindowEx(0, "scrollbar", (const char*)label, 
				style, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetScrollbarControlValue(ctrl, initValue);
			break;
		case kAOControlButton:
			style = kButtonStyle;
			if (isDefault)
				style |= BS_DEFPUSHBUTTON;
			ctrl = CreateWindowEx(0, "Button", (const char*)label, 
				style, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			break;
		case kAOControlRadio:
			ctrl = CreateWindowEx(0, "Button", (const char*)label, 
				kRadioStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			SetPlatformCheckOrRadioValue(ctrl, initValue);
			break;
		case kAOControlCheckbox:
			ctrl = CreateWindowEx(0, "Button", (const char*)label, 
				kCheckBoxStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			SetPlatformCheckOrRadioValue(ctrl, initValue);
			break;
		case kAOControlStaticText:
			ctrl = CreateWindowEx(0, "Static", (const char*)label, 
				kStaticTextStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			break;
		case kAOControlIcon:
			ctrl = CreateWindowEx(0, "Static", (const char*)label, 
				kIconStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			HICON icn = LoadIcon(gAppInstance, MAKEINTRESOURCE(resourceRef));
			SendMessage(ctrl, STM_SETIMAGE, IMAGE_ICON, (long)icn);
			break;
		case kAOControlPicture:
			ctrl = CreateWindowEx(0, "Static", (const char*)label, 
				kPictureStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			HBITMAP bmp = LoadBitmap(gAppInstance, MAKEINTRESOURCE(resourceRef));
			SendMessage(ctrl, STM_SETIMAGE, IMAGE_BITMAP, (long)bmp);
			break;
		case kAOControlFramedBox:
			ctrl = CreateWindowEx(0, "Button", (const char*)label, 
				kFrameStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			break;
		case kAOControlEditText:
			ctrl = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", "", 
				kEditTextStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			SetEditTextText(ctrl, *initialText);
			break;
		case kAOControlListBox:
			ctrl = CreateWindowEx(WS_EX_CLIENTEDGE, "ListBox", "", 
				kListBoxStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			break;
		case kAOControlPulldown:
			ctrl = CreateWindowEx(0, "ComboBox", "", 
				kPullDownStyle, 
				newRect.left, 
				newRect.top, 
				newRect.right - newRect.left, 
				newRect.bottom - newRect.top, 
				parent, 0, gAppInstance, 0);
			SetWindowLong(ctrl, GWL_ID, commandID);
			break;
		default:
			PortAssert(false); // Ooops...
			break;
		}		
	SendMessage(ctrl, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT), TRUE);
	return ctrl;
	}

void SetScrollbarControlRange(platformControl theBar, sInt16 minVal, sInt16 maxVal)
	{
	SetScrollRange(theBar, SB_CTL, minVal, maxVal, TRUE);
	}

void SetScrollbarControlValue(platformControl theBar, sInt16 theVal)
	{
	SetScrollPos(theBar, SB_CTL, theVal, TRUE);
	}

sInt16	GetScrollbarControlValue(platformControl theBar)
	{
	return GetScrollPos(theBar, SB_CTL);
	}

sInt16	GetPlatformCheckOrRadioValue(platformControl theControl)
	{
	return SendMessage(theControl, BM_GETCHECK, 0, 0);
	}
	
void	SetPlatformCheckOrRadioValue(platformControl theControl, sInt16 theValue)
	{
	SendMessage(theControl, BM_SETCHECK, theValue, 0);
	}
	
void	MovePlatformControl(platformControl theControl, platformRect* newLoc)
	{
	MoveWindow(theControl, newLoc->left, newLoc->top, newLoc->right - newLoc->left,
							newLoc->bottom - newLoc->top, TRUE);
	}
	
void	MoveScrollbarControl(platformControl theControl, platformRect* newLoc)
	{
	MovePlatformControl(theControl, newLoc);
	}
	
void	DisablePlatformControl(platformControl theControl)
	{
	EnableWindow(theControl, false);
	}
	
void	EnablePlatformControl(platformControl theControl)
	{
	EnableWindow(theControl, true);
	}

void	HidePlatformControl(platformControl theControl)
	{
	ShowWindow(theControl, SW_HIDE);
	}
	
void	ShowPlatformControl(platformControl theControl)
	{
	ShowWindow(theControl, SW_SHOW);
	}
	
void	SetEditTextText(platformControl theControl, const PortString& theText)
	{
	SetWindowText(theControl, (const char*)theText);
	}
	
void	GetEditTextText(platformControl theControl,	PortString& theText)
	{
	sInt16 len = GetWindowTextLength(theControl);
	char buffer[512];
	GetWindowText(theControl, buffer, len+1);
	theText = buffer;
	}

void	SetStaticTextText(platformControl theControl, const PortString& theText)
	{
	SetWindowText(theControl, (const char*)theText);
	}

sInt16	CountListBoxItems(platformControl theList)
	{
	sInt16 returnVal = 0;
	returnVal = SendMessage(theList, LB_GETCOUNT, 0, 0);
	return returnVal;
	}
	
sInt16	GetListBoxSelection(platformControl theList)
	{
	sInt16 returnVal = 0;
	returnVal = SendMessage(theList, LB_GETCURSEL, 0, 0);
	if (returnVal == LB_ERR)
		returnVal = kAONoSelection;
	return returnVal;
	}
	
PortString	GetTextOfListBoxItem(platformControl theList, sInt16 theIndex)
	{
	char buffer[256];
	PortString returnVal;
	SendMessage(theList, LB_GETTEXT, theIndex, (long)&buffer[0]);
	returnVal = buffer;
	return returnVal;
	}
	
void	DeleteListBoxItem(platformControl theList, sInt16 theIndex)
	{
	SendMessage(theList, LB_DELETESTRING, theIndex, 0);
	SendMessage(theList, LB_SETCURSEL, theIndex, 0);
	}
	
void	AddListBoxItem(platformControl theList, sInt16 atPos, const PortString& theItem)
	{
	SendMessage(theList, LB_INSERTSTRING, atPos, (long)((const char*)theItem));
	}

void	SelectListBoxItem(platformControl theList, sInt16 theIndex)
	{
	SendMessage(theList, LB_SETCURSEL, theIndex, 0);
	}

sInt16	CountMenuControlItems(platformControl theMenu)
	{
	sInt16 returnVal = 0;
	returnVal = SendMessage(theMenu, CB_GETCOUNT, 0, 0);
	return returnVal;
	}
	
sInt16	GetMenuControlSelection(platformControl theMenu)
	{
	sInt16 returnVal = 0;
	returnVal = SendMessage(theMenu, CB_GETCURSEL, 0, 0);
	if (returnVal == CB_ERR)
		returnVal = kAONoSelection;
	return returnVal;
	}
	
PortString	GetTextOfMenuControlItem(platformControl theMenu, sInt16 theIndex)
	{
	char buffer[256];
	PortString returnVal;
	SendMessage(theMenu, CB_GETLBTEXT, theIndex, (long)&buffer[0]);
	returnVal = buffer;
	return returnVal;
	}
	
void	DeleteMenuControlItem(platformControl theMenu, sInt16 theIndex)
	{
	SendMessage(theMenu, CB_DELETESTRING, theIndex, 0);
	SendMessage(theMenu, CB_SETCURSEL, theIndex, 0);
	}
	
void	AddMenuControlItem(platformControl theMenu, sInt16 atPos, const PortString& theItem)
	{
	SendMessage(theMenu, CB_INSERTSTRING, atPos, (long)((const char*)theItem));
	}

void	SelectMenuControlItem(platformControl theMenu, sInt16 theIndex)
	{
	SendMessage(theMenu, CB_SETCURSEL, theIndex, 0);
	}
	
} // namespace PortableViews;


#endif
