//
//	PortDialog.h

//@class:PortDialog
//@parent:Layout
//@descr
//	This class decribes a movable modal dialog box.
////
//	Uses a constructor that takes an PortDialogControlList and a dialog
//  information structure.  These can be provided by your application directly, or (more likely) loaded from
//  an XML Dialog Manager.  They're primarily data-driven; the XML file
//  or control list provides enough information for the dialog to automatically handle basic dialogs, and
//  to do most of the work for more complex ones.  Where the automatic handling is sufficient, no subclass of
//  PortDialog is necessary.
//
//	With the LayoutControlList mechanism, the "item ids" (PortDialogItem)
//  are just the command code specified in the item's list.
//
//	A note on auto-formatting dialogs:  If you use the XML LAYOUT feature to make your dialogs
// 	auto-formatting, (or if you're brave and try to construct the PortDialogLayoutNode field of
//  the PortDialogInfo yourself), the actual layout is called from DoModal(), just <strong>before</strong>
//	creating the dialog itself.  Measurement of controls is done at that time, from the info in the
// 	PortDialogControlList, <strong>not from any information you may place in the control during the
//  InitialSetup() call</strong>, since by then the controls have already been created in place.  This
//	means that if the information in the XML file isn't the final information for the control (say,
//	you leave a placeholder in for a runtime-determined string), the measurement will be wrong if you
//	wait until InitialSetup() to fill it it.  The correct way to deal with this is to substitute the
//	string <strong> in the PortDialogControlList</strong> before passing it to the PortDialog constructor
//	(or, if you're subclassing and can access the list later, at least before calling DoModal()).
//	Remember, this isn't an issue if the strings in the XML file are correct.
//@endc

#ifndef	CPF_DIALOG
#define	CPF_DIALOG

#include <vector>

#include "Window.h"

namespace PortableViews {

// A command code as appropriate for the dialog model.
typedef		LayoutItem 	PortDialogItem; 


class	PortDialog : public PortWindow {
friend class AVXMLDialogManager;
protected:
	resID				iResID;
	bool				iExiting;
	sInt16				iExitValue;
	PortDialogInfo		iInfo;

#if (UT_TARGET_WIN32)
public:	
				void	SetupControls(void);
#endif
public:
		// Inherited from Layout	
		virtual void	LayoutControls(LayoutNode& nd);

		 				PortDialog(LayoutControlList& controls, PortDialogInfo& info);
		void			SetWindow(platformWindow theWind) { iWindow = theWind; }
		PortDialogItem 	DoModal(void);
		
		void			CloseDialog(PortDialogItem result, bool onlyIfValid = true);

virtual void	ControlValueChanged(platformControl /*theControl*/, LayoutItem theItem) { ItemHit(theItem); }
		
	// Subclass overrides
	virtual bool		ItemHit(PortDialogItem theItem);
	virtual bool		KeyHit(char /*theChar*/) { return false; }
	virtual void		InitialSetup(void);
	virtual void		LastChance(PortDialogItem /*exitVal*/) {}
	virtual bool		ValidData(void) { return true; }
	virtual void		IdleTime(void) {}
};
} // namespace PortableViews;

#endif