//
//
//	This file defines error-handling methods and macros.  This is not a primary
//  focus of the Dialog handling code, so you can ignore it fairly effectively.
//
#include "UniversalSymbols.h"

#include <string.h>
#include <stdio.h>

#if (UT_TARGET_MACOS)
#include <Dialogs.h>
#endif

#include "PortErrors.h"
	
namespace PortableViews {
const sInt16 kMaxLineSize							= 14;
const sInt16 kOtherOverhead							= 2;
const char * const kFailedMessage					= "Assertion failed (file/line/expression) : ";

void PortAssertFailed (const char * const theFile, sInt32 theLine, const char * const expression);

//
//	Name:	__AssertFailed
//
//	Descr:	Outputs a message pinpointing the file and line of an unsupported assertion.
//
void PortAssertFailed (const char * const theFile, sInt32 theLine, const char * const expression) 
{
	char *					outString = new char   [strlen(theFile) +
													strlen(expression) +
													kMaxLineSize + kOtherOverhead +
													strlen(kFailedMessage) + 3];

	sprintf(&outString[1],  "%s%s/%d/%s\n", kFailedMessage, theFile, theLine, expression);
#if (UT_TARGET_MACOS)
	outString[0] = static_cast<char>(strlen(&outString[1])-1);
	DebugStr((unsigned char*)outString);
#else if (UT_TARGET_WIN32)
	MessageBox(NULL, &outString[1], "Assertion Failed!", MB_ABORTRETRYIGNORE);
#endif
}

//
//	Name:	UtException
//
//	Descr:	Exeception constructor.
//
UtException::UtException(errorResult err, char* text, char* file, uInt32 line)
	{
	iError = err;
	iLine = line;
	strncpy(iText, text, 79);
	strncpy(iFile, file, 79);
	}
	
//
//	Name:	Display
//
//	Descr:	Display a dialog box or alert indicating the problem.
//
void	UtException::Display(void)
	{
	char message[256];
	sprintf(&message[1], "Exception Caught: err: %lu, \"%s\"  File: \"%s\"   Line: %d",
			iError, iText, iFile, iLine);
	
	// Okay, now we've got to do something with it.
#if (UT_TARGET_MACOS)
	message[0] = (unsigned char)std::strlen(&message[1]);
	DebugStr((unsigned char*)message);
#else if (UT_TARGET_WIN32)
	MessageBox(NULL, message, "PortableViews Exception", MB_YESNO);
#endif
	}

} // namespace PortableViews;