#if (UT_TARGET_WIN32)

// Define a bunch of abstract types and variables into their Windows equivalents.
// This makes portable code easier.

// Get some windows types, but only for this file.
#ifdef __MWERKS__
	#include "ansi_prefix.win32.h"
#endif

// Define this symbol to prevent Microsoft's definition of min & max from conflicting
// with the standard lib's 
#define NOMINMAX

#include <Windows.h>
#include <commctrl.h>

extern HINSTANCE 	gAppInstance; 	// Application Instance
extern HWND 	  	gAppWindow;		// Main application Window

namespace PortableViews {

// Universal Types File
typedef unsigned char 		uByte;
typedef signed char			sByte;

typedef unsigned char		uInt8;
typedef signed char			sInt8;
typedef signed char			int8;

typedef unsigned short		uInt16;
typedef short				sInt16;
typedef short				int16;

typedef unsigned long		uInt32;
typedef long				sInt32;
typedef long				int32;

typedef unsigned __int64	uInt64;
typedef signed	__int64		sInt64;
typedef sInt64				int64;

typedef	double				loFloat;
typedef long double			hiFloat;

typedef	void*				universalPtr;

typedef	long	needs_def; //FIX:: Anything defined to this needs to get defined.

typedef HENHMETAFILE		platformMetaFile;

struct	platformPoint {
	int16	x;
	int16	y;
	};
	
struct	platformLongPoint {
	int32	x;
	int32	y;
	};

// A rectangle, and a whole slew of accessor functions for it.
typedef RECT		platformRect;
inline void SetRectLeft(platformRect* theRect, int16 val) { theRect->left = val; }
inline void SetRectRight(platformRect* theRect, int16 val) { theRect->right = val; }
inline void SetRectTop(platformRect* theRect, int16 val) { theRect->top = val; }
inline void SetRectBottom(platformRect* theRect, int16 val) { theRect->bottom = val; }
inline void SetRectTLBR(platformRect* theRect, int16 t, int16 l, int16 b, int16 r)
			{ theRect->top = t; theRect->bottom = b; theRect->left = l; theRect->right = r; }
inline void SetRectLTRB(platformRect* theRect,  int16 l, int16 t, int16 r, int16 b)
			{ theRect->top = t; theRect->bottom = b; theRect->left = l; theRect->right = r; }

inline int16 GetRectLeft(platformRect* theRect) { return theRect->left; }
inline int16 GetRectRight(platformRect* theRect) { return theRect->right; }
inline int16 GetRectTop(platformRect* theRect) { return theRect->top; }
inline int16 GetRectBottom(platformRect* theRect) { return theRect->bottom; }
	
// A color spec, and a whole slew of accessor functions for it, too
struct	platformRGBColor
	{
	uInt8	red;
	uInt8	green;
	uInt8	blue;
	uInt8	alpha;
	};
	
inline void	SetColorRed(platformRGBColor* color, uInt8 val) { color->red = (((unsigned short)val)<<8)+val; }	
inline void	SetColorGreen(platformRGBColor* color, uInt8 val) { color->green = (((unsigned short)val)<<8)+val; }	
inline void	SetColorBlue(platformRGBColor* color, uInt8 val) { color->blue = (((unsigned short)val)<<8)+val; }	
inline void	SetColorRGB(platformRGBColor* color, uInt8 r, uInt8 g, uInt8 b) 
	{ 	
	color->red = (((unsigned short)r)<<8)+r; 
	color->green = (((unsigned short)g)<<8)+g; 
	color->blue = (((unsigned short)b)<<8)+b; 
	}	

typedef HWND	platformWindow;
typedef WNDPROC platformWindowProc;
	
struct	platformDC {
	HDC	    dc;
	HWND	win;
	};
	
typedef ENUMLOGFONTEX	platformTextSpecs;

typedef	HMENU	platformMenu;
typedef	HMENU	platformMenuBar;

typedef TEXTMETRIC platformFontMetrics;

inline double 	GetMetricAscent(const platformFontMetrics& fm) { return fm.tmAscent; }
	
inline double	GetMetricDescent(const platformFontMetrics& fm) { return fm.tmDescent; }

typedef	HICON	platformIcon;

typedef	HCURSOR	platformCursor;

typedef	HWND	platformControl;

typedef	HIMAGELIST	platformMaskableBitmap;

typedef	HIMAGELIST	platformBitmap;
typedef	HBITMAP		platformOffscreen;

typedef	HRGN	platformRgn;
typedef	HRGN	platformPoly;

typedef	HANDLE			platformOpenFileHandle;

typedef	char*				CStringPtr;
typedef const char * const	constCStringPtr;

typedef unsigned char*		PStringPtr;
typedef const unsigned char* const constPStringPtr;

typedef sInt32 				charLong;
typedef sInt16 				resID;

typedef	unsigned char		PString31[32];
typedef unsigned char		PString255[256];

} // namespace PortableViews
#endif
