//
//	PortWindow.h

// This class is being provided because it implements some of the dialog functionality.
// It's been heavily hacked to elimintate a dependence on the rest of an application
// framework.  Enough remains to use as a rudimentary window class if you like, but this
// is not the focus of the provided code.

#ifndef 	AV_WINDOW
#define		AV_WINDOW

#include "PortString.h"
#include "Layout.h"
#include <vector>
#include <map>

#ifdef UT_USES_OPENGL
#if (UT_TARGET_MACOS)
	#include "agl.h"
#endif
#include "GL/gl.h"
#endif

namespace PortableViews {
class AVWidget;
class AOFileStructure;


class	PortWindow : public Layout {
friend class AVXMLDialogManager;
	static	long								sStaggerCount;
public: // Public because the dialog proc on Windows needs to get to it.
	static	std::map<platformWindow, PortWindow*>	sWindowMap;
protected:
	platformWindow				iWindow;
	std::vector<AVWidget*>		iWidgets;
	bool						iAllowResize;
	void*						iUserData;
	sInt16 						iDefault; // used only for "dialog-like" windows
	sInt16						iCancel;  // used only for "dialog-like" windows
	bool						iIsDialog;

// A couple of helper functions here, to do window-related things.
public:
static	void	RegisterWindowClasses(platformWindowProc p, platformWindowProc f, platformWindowProc m);	
static  platformWindow	CreatePlatformWindow(void);
#if (UT_TARGET_MACOS)
static  platformWindow	CreatePlatformDialog(void);
#endif
static	PortWindow*	PlatformWindowToPortWindow(platformWindow win);
static  PortWindow*	GetFrontmostWindow(void); 
static  PortWindow*	GetFrontmostDocumentWindow(void); 
	
private:
		void	SubConstructWindow(platformWindow theWin);
protected:
				PortWindow(void) { /* used by PortDialog */ }
		void 	CreateActualControls(void);	
public:
				PortWindow(platformWindow theWin);
		virtual	~PortWindow();
		platformWindow GetPlatformWindow(void) { return iWindow; }
		bool	CanResize(void) { return iAllowResize; }
		void	Resize (sInt16 h, sInt16 v);
		void	Move(int16 h, int16 v);
		void	Hide(void);
		void	Show(void);
		void	SetTitle(const PortString& title);
		void	Update(void);
		void	Invalidate(platformRect* rect = NULL);
		void	ForceRedraw(void);
		void	Validate(platformRect* rect = NULL);
		void	GetWindowContextSize(sInt16* x, sInt16* y);
		void	GetWindowContextRect(platformRect* theRect);
		void	Stagger(void);
		
		// User Data functions
		void	SetUserData(void* ud) { iUserData = ud; }
		void*	GetUserData(void) { return iUserData; }

		// Overrides
virtual bool	OKToClose(void) { return true; }
virtual	void	Close(void);
virtual void	PostResize (sInt16 h, sInt16 v);
		
		// Control handling functions
virtual sInt16	ScrollButtonAmount(platformControl /*theControl*/) { return 16; }
virtual sInt16	ScrollPageAmount(platformControl /*theControl*/ ) { return 160; }
virtual void	ControlValueChanged(platformControl /*theControl*/, LayoutItem /*theItem*/) {}
virtual void	ListBoxDoubleClicked(platformControl /*theControl*/, LayoutItem /*theItem*/) {}

		void			SetDefaultButton(LayoutItem val);
		sInt16			GetDefaultButton(void) { return iDefault; }
		void			SetCancelButton(LayoutItem val);
		sInt16			GetCancelButton(void) { return iCancel; }

		sInt32			GetControlNumericValue(LayoutItem theItem);
		PortString		GetControlTextValue(LayoutItem theItem);

		void			SelectLayoutItem(LayoutItem val);
		void			EnableLayoutItem(LayoutItem val, bool enable);
		void			SetFocusToLayoutItem(LayoutItem val);
		
		void			SetRadioButtonRange(LayoutItem first, LayoutItem last, LayoutItem on);
		
		// Get the control for the specified item.  Works for
		// controls, radio buttons, checkboxes, and buttons.
		platformControl	GetLayoutControl(LayoutItem theItem);
		LayoutItem	GetControlLayoutItem(platformControl theControl);
		
		void			CreateControlsFromLayout(LayoutControlList& controls, PortDialogInfo& info,
												sInt16 hMargin, sInt16 vMargin, bool sizeToFit);
};

} // namespace PortableViews;

#endif