//
//	PortWindow.cp

// This class is being provided because it implements some of the dialog functionality.
// It's been heavily hacked to elimintate a dependence on the rest of an application
// framework.  Enough remains to use as a rudimentary window class if you like, but this
// is not the focus of the provided code.
//
//	This file is Mac-specific.
//
#include "UniversalSymbols.h"

#if (UT_TARGET_MACOS)
#include <CarbonEvents.h>

#include "Controls.h"
#include "PortString.h"
#include "Window.h"
#include <MacWindows.h>
#include <ControlDefinitions.h>
#include <CodeFragments.h>

using namespace PortableViews;

namespace PortableViews {
long 	PortWindow::sStaggerCount = 0;
}


namespace {

// We need to create a "user event" to get change notifications on edit text fields.
// There are slightly easier ways to do this, if you aren't bothered by that pesky "incorrect
// functionality" thing.  Hopefully Apple will correct this oversight soon.
const uInt32 kUserControlClass = FOUR_CHAR_CODE('USRc');
const uInt32 kUserControlChangeKind = FOUR_CHAR_CODE('UCCh');
const OSType kChangeName = FOUR_CHAR_CODE('CChn');
const OSType kChangeParam = FOUR_CHAR_CODE('CCng');

struct AVControlChangeNotifier {
	PortWindow* 	winPtr;
	ControlRef	control;
	LayoutItem code;
};

const OSType kEventParamPostTarget = FOUR_CHAR_CODE('ptrg');
const OSType typeEventTargetRef = FOUR_CHAR_CODE('etrg');

pascal ControlKeyFilterResult EditTextFilter(ControlRef theControl, SInt16* /*keyCode*/, 
				SInt16* /*charCode*/, EventModifiers* /*modifiers*/)
	{
	WindowRef win = GetControlOwner(theControl);
	PortWindow* targetWin = PortWindow::PlatformWindowToPortWindow(win);
	if (targetWin != NULL)
		{
		LayoutControlList& theList = targetWin->GetControlList();
		LayoutItem id = 0;
		bool found = false;
		long n = theList.size();
		for (long i = 0; i < n; i++)
			if (theList[i].iControl == theControl)
				id = theList[i].iCommandID;

		EventRef newEvent;
		OSStatus err = MacCreateEvent(NULL, kUserControlClass, kUserControlChangeKind, GetCurrentEventTime(), 0, &newEvent);
		AVControlChangeNotifier ccn;
		ccn.winPtr = targetWin;
		ccn.code = id;
		ccn.control = theControl;
		EventTargetRef winTarget = GetWindowEventTarget(win);
		err = SetEventParameter(newEvent, kChangeName, kChangeParam,  sizeof(AVControlChangeNotifier), &ccn);
		err = SetEventParameter(newEvent, kEventParamPostTarget, typeEventTargetRef, sizeof(EventTargetRef),
				&winTarget);
		err = PostEventToQueue(GetMainEventQueue(), newEvent, kEventPriorityStandard);
		ReleaseEvent(newEvent);
		}
	
	return kControlKeyFilterPassKey;
	}

pascal void EditTextValidator(ControlRef theControl)
	{
	WindowRef win = GetControlOwner(theControl);
	PortWindow* targetWin = PortWindow::PlatformWindowToPortWindow(win);
	if (targetWin != NULL)
		{
		LayoutControlList& theList = targetWin->GetControlList();
		bool found = false;
		long n = theList.size();
		for (long i = 0; i < n; i++)
			{
			if (theList[i].iControl == theControl)
				{
				found = true;
				targetWin->ControlValueChanged(theControl, theList[i].iCommandID);
				}
			}
		if (!found)
			targetWin->ControlValueChanged(theControl, 0);
		}
	}

// Carbon event handler for Windows.  It's fairly monolithic, but it turns out there's relatively
// few (about a half dozen) events that we need to handle generically.  For most events, the
// default handling will take care of it completely, or else convert it into ones we're interested in.
pascal OSStatus 	WindowEventHandler(EventHandlerCallRef inHandlerCallRef, EventRef inEvent, void *inUserData)
	{
	OSStatus returnVal = eventNotHandledErr;
	
	uInt32 eventClass = GetEventClass(inEvent);
	uInt32 eventKind = GetEventKind(inEvent);
	platformWindow platWin = reinterpret_cast<platformWindow>(inUserData);
	PortWindow* targetWin = PortWindow::PlatformWindowToPortWindow(platWin);
	
	if (eventClass == kEventClassKeyboard && eventKind == kEventRawKeyDown)
		{
		uInt32 keyCode;
		GetEventParameter(inEvent, kEventParamKeyCode, typeUInt32, NULL, sizeof(keyCode), NULL, &keyCode);
		}
	else if (eventClass == kUserControlClass && eventKind == kUserControlChangeKind)
		{
		AVControlChangeNotifier ccn;
		GetEventParameter(inEvent, kChangeName, kChangeParam, NULL, sizeof(ccn), NULL, &ccn);
		ccn.winPtr->ControlValueChanged(ccn.control, ccn.code);
		}
	else if (eventClass == kEventClassWindow && eventKind == kEventWindowDrawContent)
		{
		CallNextEventHandler(inHandlerCallRef, inEvent);
		returnVal = noErr;
		}
	else if (eventClass == kEventClassWindow && eventKind == kEventWindowHandleContentClick)
		{
		returnVal = CallNextEventHandler(inHandlerCallRef, inEvent);
		}
	else if (eventClass == kEventClassWindow && eventKind == kEventWindowResizeCompleted)
		{
		if (targetWin != NULL)
			{
			sInt16 x;
			sInt16 y;
			targetWin->GetWindowContextSize(&x, &y);
			targetWin->PostResize(x, y);
			}
		// In any case, call the next handler.
		CallNextEventHandler(inHandlerCallRef, inEvent);
		returnVal = noErr;
		}
	else if ((eventClass == kEventClassControl && eventKind == kEventControlHit) ||
			(eventClass == kEventClassControl && eventKind == kEventControlValueFieldChanged))
		{
		ControlRef theControl;
		GetEventParameter(inEvent, kEventParamDirectObject, typeControlRef, NULL, 
								sizeof(ControlRef), NULL, &theControl);
		bool found = false;
		LayoutControlList& theList = targetWin->GetControlList();
		long n = theList.size();
		for (long i = 0; i < n; i++)
			{
			if (theList[i].iControl == theControl)
				{
				found = true;
				targetWin->ControlValueChanged(theControl, theList[i].iCommandID);
				}
			}
		if (!found)
			targetWin->ControlValueChanged(theControl, 0);
		// Leave the return value alone, other things might want it.
		}
	else
		{
		PortAssert(false); // Whoops, we should be processing everything we ask for.
		}
	return returnVal;
	}
}

namespace PortableViews {

//STATIC
platformWindow	PortWindow::CreatePlatformWindow(void)
	{
	platformRect	wRect;
	SetRectTLBR(&wRect, 0, 30, 430, 600);
	PortString name("Untitled");
	platformWindow theWindow;
	OSStatus err;
	WindowAttributes attr = kWindowNoAttributes;
	attr = kWindowStandardDocumentAttributes;   
	err = CreateNewWindow(kDocumentWindowClass, attr, &wRect, &theWindow);
		
	InstallStandardEventHandler(GetWindowEventTarget(theWindow));
    EventHandlerUPP theHandler = NewEventHandlerUPP(WindowEventHandler);
    EventTypeSpec	handledEvents[] = { 
    									{kUserControlClass, kUserControlChangeKind},
    									{kEventClassWindow, kEventWindowDrawContent},
    									{kEventClassWindow, kEventWindowResizeCompleted},
    									{kEventClassWindow, kEventWindowHandleContentClick},
										{kEventClassControl, kEventControlHit},
										{kEventClassControl, kEventControlValueFieldChanged},
										{kEventClassKeyboard, kEventRawKeyDown}
									   };

    InstallWindowEventHandler(theWindow, theHandler, 7, handledEvents,
    					 reinterpret_cast<void *>(theWindow), NULL);
  	return theWindow;
	}

//STATIC
platformWindow	PortWindow::CreatePlatformDialog(void)
	{
	platformRect	wRect;
	SetRectTLBR(&wRect, 0, 30, 430, 600);
	PortString name("Untitled");
	platformWindow theWindow;
	OSStatus err;
	WindowAttributes attr = kWindowNoAttributes;
	err = CreateNewWindow(kMovableModalWindowClass, attr, &wRect, &theWindow);
		
	InstallStandardEventHandler(GetWindowEventTarget(theWindow));
    EventHandlerUPP theHandler = NewEventHandlerUPP(WindowEventHandler);
    EventTypeSpec	handledEvents[] = { 
    									{kUserControlClass, kUserControlChangeKind},
    									{kEventClassWindow, kEventWindowDrawContent},
    									{kEventClassWindow, kEventWindowResizeCompleted},
    									{kEventClassWindow, kEventWindowHandleContentClick},
										{kEventClassControl, kEventControlHit},
										{kEventClassControl, kEventControlValueFieldChanged},
										{kEventClassKeyboard, kEventRawKeyDown}
									   };

    ControlHandle toss;
	CreateRootControl(theWindow, &toss);
	InstallWindowEventHandler(theWindow, theHandler, 7, handledEvents,
    					 reinterpret_cast<void *>(theWindow), NULL);
  	return theWindow;
	}


//STATIC
void	PortWindow::RegisterWindowClasses(platformWindowProc /*p*/, platformWindowProc /*f*/, platformWindowProc /*m*/)
	{
	// Does nothing, rather elegantly.
	}


//
//	Name:	PortWindow constructor.  
//
//	Descr: 	Create the window and the "display context" for it.  On the mac, the DC
//			is just a record containing a GrafPtr.  Because of the immense differences
//			in how windows are created and allocated on Mac vs. PC, we defer the creation
//			of the window to the framework layer, and require that an already-constructed
//			one be passed in here.
//	
PortWindow::PortWindow(platformWindow theWin)
	{
	SubConstructWindow(theWin);
	}
		
void	PortWindow::SubConstructWindow(platformWindow theWin)
	{
	platformDC theDC;
	iWindow = theWin;
	theDC.isWindow = true;
	theDC.theGrafPtr = GetWindowPort(iWindow);
	theDC.theWindow = iWindow;
	SetWRefCon(theWin, reinterpret_cast<long>(this));
	iUserData = 0;
	
	sWindowMap[theWin] = this;

#ifdef UT_USES_OPENGL
	iOpenGLWindow = false;
	iAccelerated = false;
	if (aglChoosePixelFormat == (void *)kUnresolvedCFragSymbolAddress)
		OneDialog("OpenGL is not available.");
#endif
	}

#ifdef UT_USES_OPENGL

void	PortWindow::PrepareForOpenGL(void)
	{
	GrafPtr savePort;
	GetPort(&savePort);
	iAccelerated = true;
	GLenum error;
	long attrib[] = { AGL_RGBA, AGL_ACCELERATED, AGL_DOUBLEBUFFER, AGL_DEPTH_SIZE, 16, AGL_NONE };
	long attrib2[] = { AGL_RGBA, AGL_DOUBLEBUFFER, AGL_DEPTH_SIZE, 16, AGL_NONE };
	AGLPixelFormat fmt;

	SetPort(GetWindowPort(iWindow));
	
	// If possible, get an accelerated context.
	fmt = aglChoosePixelFormat(NULL, 0, attrib);
	// Otherwise, we'll have to settle for software ones
	if (fmt == NULL)
		{
		iAccelerated = false;
		fmt = aglChoosePixelFormat(NULL, 0, attrib2);
		}
		
	if (fmt == NULL)
		{
		// Can't even get an unaccelerated one.  Hmmm.
		OneDialog("Can't Get an OpenGL Context"); //FIX:: Real error handling would be nice
		}
	else
		{
		error = aglGetError();
		iOpenGLContext = aglCreateContext(fmt, NULL);
		error = aglGetError();
		aglDestroyPixelFormat(fmt);
		error = aglGetError();
		}
	SetPort(savePort);
	}
	
void	PortWindow::MakeCurrentOpenGLContext(void)
	{
	GLenum error;
	SetPort(GetWindowPort(iWindow));
	aglSetDrawable(iOpenGLContext, (AGLDrawable)GetWindowPort(iWindow));
	error = aglGetError();
	aglSetCurrentContext(iOpenGLContext);
	error = aglGetError();
	iOpenGLWindow = true;
	error = aglGetError();
 	}
	
void	PortWindow::ReleaseOpenGLContext(void)
	{
	iOpenGLWindow = false;
	// make the rendering context not current 
	aglSetCurrentContext(NULL) ; 
	 
	// delete the rendering context 
	aglDestroyContext(iOpenGLContext);
	}
#endif	

void	PortWindow::PostResize (sInt16 /*h*/, sInt16 /*v*/) 
	{
	}


PortWindow::~PortWindow(void)
	{
	DisposeWindow(iWindow);
	sWindowMap.erase(iWindow);
	}

void 	PortWindow::Move(int16 h, int16 v)
	{
	MoveWindow(iWindow, h, v, true);
	}

//
//	Name:	Invalidate
//
//	Descr:	Mark the specified rectangle (or the whole window if NULL)
//		as invalid.  Update/Paint events will be generated at the next
//		appropriate time to repaint the invalid region.
//
void	PortWindow::Invalidate(platformRect* rect)
	{
	GrafPtr	savePort;
	GetPort(&savePort);
	SetPort(GetWindowPort(iWindow));
	if (rect == NULL)
		{
		Rect invRect;
		GetPortBounds(GetWindowPort(iWindow), &invRect);
		InvalWindowRect(iWindow, &invRect);
		}
	else
		InvalWindowRect(iWindow, rect);
	SetPort(savePort);
	}
	
//
//	Name:	Validate
//
//	Descr:	Mark the specified rectangle (or the whole window if NULL)
//		as valid (not needing repainting).   
//
void	PortWindow::Validate(platformRect* rect)
	{
	GrafPtr	savePort;
	GetPort(&savePort);
	SetPort(GetWindowPort(iWindow));
	if (rect == NULL)
		{
		Rect vRect;
		GetWindowBounds(iWindow, kWindowContentRgn, &vRect);
		ValidWindowRect(iWindow, &vRect);
		}
	else
		ValidWindowRect(iWindow, rect);
	SetPort(savePort);
	}

	
void	PortWindow::Update(void)
	{
	GrafPtr	savePort;
	GetPort(&savePort);
	SetPort(GetWindowPort(iWindow));
	ThemeDrawingState tds;
	GetThemeDrawingState(&tds);
	BeginUpdate(iWindow);
	RgnHandle vRgn = NewRgn();
	GetWindowRegion(iWindow, kWindowContentRgn, vRgn);
	UpdateControls(iWindow, vRgn);
	DisposeRgn(vRgn);
	if (iAllowResize)
		{
		RgnHandle rgn = NewRgn();
		GetClip(rgn);
		platformRect r;
		GetWindowBounds(iWindow, kWindowContentRgn, &r);
		r.left = r.right - 15;
		r.top = r.bottom - 15;
		ClipRect(&r);
		DrawGrowIcon(iWindow);
		SetClip(rgn);
		DisposeRgn(rgn);
		}
	EndUpdate(iWindow);
	SetThemeDrawingState(tds, true);
	SetPort(savePort);
	}
		
void	PortWindow::ForceRedraw(void)
	{
	Invalidate();
	Update();
	QDFlushPortBuffer(GetWindowPort(iWindow), NULL);
	}
	
void	PortWindow::Resize(sInt16 h, sInt16 v)
	{
	if (CanResize())
		ChangeWindowAttributes(iWindow, kWindowResizableAttribute, kWindowNoAttributes);
	else
		ChangeWindowAttributes(iWindow, kWindowNoAttributes, kWindowResizableAttribute);
	SizeWindow(iWindow, h, v, true);
	PostResize(h, v);
	}
	
void	PortWindow::Hide(void)
	{
	HideWindow(iWindow);
	}
	
void	PortWindow::Show(void)
	{
	if (CanResize())
		ChangeWindowAttributes(iWindow, kWindowResizableAttribute, kWindowNoAttributes);
	else
		ChangeWindowAttributes(iWindow, kWindowNoAttributes, kWindowResizableAttribute);
	ShowWindow(iWindow);
	}
	
void	PortWindow::SetTitle(const PortString& title)
	{
	SetWTitle(iWindow, title);
	}
	
void	PortWindow::Close(void)
	{
	// Override for specific behaviours, if needed.
	}

	
void	PortWindow::GetWindowContextSize(sInt16* x, sInt16* y)
	{
	Rect r;
	GetWindowBounds(iWindow, kWindowContentRgn, &r);
	*x = r.right - r.left;
	*y = r.bottom - r.top;
	}
	
void	PortWindow::GetWindowContextRect(platformRect* theRect)
	{
	theRect->left = 0;
	theRect->top = 0;
	GetWindowContextSize(&theRect->right, &theRect->bottom);
	}
	
//static  
PortWindow*	PortWindow::GetFrontmostWindow(void)
	{
	PortWindow* returnVal = PortWindow::PlatformWindowToPortWindow(FrontWindow());
	WindowRef win = FrontWindow();
	return returnVal;
	}

void	PortWindow::CreateActualControls(void)
	{
	static ControlKeyFilterUPP cFilter = NewControlKeyFilterUPP(EditTextFilter);
	static ControlEditTextValidationUPP vFilter = NewControlEditTextValidationUPP(EditTextValidator);
	
	long n = iControlList.size();
	// And set the values, default, and cancel buttons from the Control List
	for (long i = 0; i < n; i++)
		{
		ControlHandle ctl = CreatePlatformControl(iControlList[i].iType, iWindow, 
					iControlList[i].iTitle, &iControlList[i].iRect, iControlList[i].iCommandID,
					iControlList[i].iNumericValue, iControlList[i].iDefault, iControlList[i].iResourceRef, 
					&iControlList[i].iTextValue);
		switch (iControlList[i].iType)
			{
			case kAOControlEditText:
				SetControlData(ctl, kControlEntireControl, kControlEditTextKeyFilterTag, sizeof(cFilter),
									&cFilter);
				SetControlData(ctl, kControlEntireControl, kControlEditTextValidationProcTag, sizeof(vFilter),
									&vFilter);
				break;
			case kAOControlListBox:
				for (long s = 0; s < iControlList[i].iListItems.size(); s++)
					AddListBoxItem(ctl, 9999, iControlList[i].iListItems[s]);
				SelectListBoxItem(ctl, 0);
				break;
			break;
			}
		iControlList[i].iControl = ctl;
		if (iControlList[i].iDefault)
			SetDefaultButton(iControlList[i].iCommandID);
		if (iControlList[i].iCancel)
			SetCancelButton(iControlList[i].iCommandID);
		}
	DrawControls(iWindow); // In case it's visible.
	}

void	PortWindow::SetRadioButtonRange(LayoutItem first, LayoutItem last, LayoutItem on)
	{
	PortAssert(last > first);
	for (LayoutItem x = first; x <= last; x++)
		{
		platformControl ct = GetLayoutControl(x);
		if (x == on)
			SetPlatformCheckOrRadioValue(ct, 1);
		else
			SetPlatformCheckOrRadioValue(ct, 0);
		}
	}

void	PortWindow::SetDefaultButton(LayoutItem val)
	{
	iDefault = val;
	SetWindowDefaultButton(iWindow, GetLayoutControl(val));
	}

void	PortWindow::SetCancelButton(LayoutItem val)
	{
	iCancel = val;
 	SetWindowCancelButton(iWindow, GetLayoutControl(val));
	}



// Get the control for the specified item.  Works for
// controls, radio buttons, checkboxes, and buttons.
platformControl	PortWindow::GetLayoutControl(LayoutItem theItem)
	{
	return iControlList[GetControlListIndexFromCommand(theItem)].iControl;
	}


// Sets the focus to the specified item, which will only have an effect on the Mac
// if said item is edittext, in which case it will select it.
void	PortWindow::SelectLayoutItem(LayoutItem val)
	{
	platformControl ctrl = GetLayoutControl(val);
	long ind = GetControlListIndexFromCommand(val);
	if (iControlList[ind].iType == kAOControlEditText)
		{
		ControlEditTextSelectionRec sel;
		sel.selStart = 0;
		sel.selEnd = 32766;
		SetControlData(ctrl,  kControlEntireControl, kControlEditTextSelectionTag, sizeof(ControlEditTextSelectionRec), &sel);
		}
	}

// Same idea, but doesn't select, and works on any control that
// can receive keyboard focus.	
void	PortWindow::SetFocusToLayoutItem(LayoutItem val)
	{
	platformControl ctrl = GetLayoutControl(val);
	SetKeyboardFocus(iWindow, ctrl, kControlFocusNoPart);
	SetKeyboardFocus(iWindow, ctrl, kControlFocusNextPart);
	DrawControls(iWindow);
	}

void	PortWindow::EnableLayoutItem(LayoutItem val, bool enable)
	{
	platformControl ctrl = GetLayoutControl(val);
	if (enable)
		EnablePlatformControl(ctrl);
	else
		DisablePlatformControl(ctrl);
	}
}  // namespace PortableViews

#endif