//
//	PortWindowWin.cp

// This class is being provided because it implements some of the dialog functionality.
// It's been heavily hacked to elimintate a dependence on the rest of an application
// framework.  Enough remains to use as a rudimentary window class if you like, but this
// is not the focus of the provided code.
//
//	This file is Windows-specific.
//
#include "UniversalSymbols.h"

#if (UT_TARGET_WIN32)
#include <windows.h>
#include "Controls.h"
#include "Window.h"

namespace PortableViews {

long 	PortWindow::sStaggerCount = 0;

//STATIC
//@method:PortWindow::RegisterWindowClasses
//@arg:p:platformWindowProc
//  The window procedure to register (on Win32), not used on the Mac.
//@arg:f:platformWindowProc
//  The window procedure to register (on Win32) for MDI frame windows, not used on the Mac.
//@arg:m:platformWindowProc
//  The window procedure to register (on Win32) for MDI child windows, not used on the Mac.
//@descr
// [Static] This static method registers the basic PortableViews window classes
// on Win32.  You don't have to call it unless you're using the AV layer
// but not the CPF layer.  If you're using CPF, it's called for you at
// the appropriate time.
// If you do need to call this, call it once before any windows are created,
// and pass a pointer to the window procedure you're using for your application.
// The registered application window class is called "CPFWindowBase".
//@endm
void	PortWindow::RegisterWindowClasses(platformWindowProc p, platformWindowProc f, platformWindowProc m)
	{
	WNDCLASSEX  wndclass;

		
	// Register an application window type
	wndclass.cbSize        = sizeof (wndclass) ;
	wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
	wndclass.lpfnWndProc   = p ;
	wndclass.cbClsExtra    = 0 ;
	wndclass.cbWndExtra    = 0 ;
	wndclass.hInstance     = gAppInstance ;
	wndclass.hIcon         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
	wndclass.lpszMenuName  = NULL ;
	wndclass.lpszClassName = "CPFWindowBase" ;
	wndclass.hIconSm         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIconSm         = LoadIcon (NULL, IDI_APPLICATION) ;
	ATOM err = RegisterClassEx (&wndclass);
	PortAssert(err);
	
	// And a frame window type
	wndclass.cbSize        = sizeof (wndclass) ;
	wndclass.style         = 0 ;
	wndclass.lpfnWndProc   = f ;
	wndclass.cbClsExtra    = 0 ;
	wndclass.cbWndExtra    = 0 ;
	wndclass.hInstance     = gAppInstance ;
	wndclass.hIcon         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndclass.hbrBackground = (HBRUSH) (COLOR_APPWORKSPACE + 1) ;
	wndclass.lpszMenuName  = NULL ;
	wndclass.lpszClassName = "CPFWindowFrame";
	wndclass.hIconSm         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIconSm         = LoadIcon (NULL, IDI_APPLICATION) ;
	err = RegisterClassEx (&wndclass);
	PortAssert(err);

	// And an MDI Child Window Type
	wndclass.cbSize        = sizeof (wndclass) ;
	wndclass.style         = 0 ;
	wndclass.lpfnWndProc   = m ;
	wndclass.cbClsExtra    = 0 ;
    wndclass.cbWndExtra    = 256;
	wndclass.hInstance     = gAppInstance ;
	wndclass.hIcon         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
	wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
	wndclass.lpszMenuName  = NULL ;
	wndclass.lpszClassName = "CPFWindowMDIChild";
	wndclass.hIconSm         = LoadIcon (gAppInstance, MAKEINTRESOURCE(1000)) ;
	if (wndclass.hIcon == NULL)
		wndclass.hIconSm         = LoadIcon (NULL, IDI_APPLICATION) ;
	err = RegisterClassEx (&wndclass);
	PortAssert(err);
	}


//STATIC
//@method:PortWindow::CreatePlatformWindow
//@return:platformWindow
//  The newly created window.
//@arg:whichType:winType
//  The type of window to create, see PortWindow.h for a list of
//  usable types.
//@descr
// [Static] This static method creates a window of the specified type,
// and returns a platformWindow for it.  At the present time, the
// only types that are used are kAOWinApp and kAOWinDocument.
//@endm
platformWindow	PortWindow::CreatePlatformWindow(void)
	{
	static sInt32 id = 1;
	platformWindow hwnd;
   	hwnd = CreateWindow ("PortViewsWindowBase",         // window class name
		            "Application Window",     	// window caption
                    WS_OVERLAPPEDWINDOW,     // window style
                    CW_USEDEFAULT,           // initial x position
                    CW_USEDEFAULT,           // initial y position
                    CW_USEDEFAULT,           // initial x size
                    CW_USEDEFAULT,           // initial y size
                    NULL,                    // parent window handle
                    NULL,                    // window menu handle
                    gAppInstance,            // program instance handle
		            NULL) ;		             // creation parameters		
	SetWindowLong(hwnd, GWL_USERDATA, 0);
	return hwnd;
	}

//@method:PortWindow::PortWindow
//@arg:theWin:platformWindow
//  The platform window to associate with this PortWindow.
//@arg:whichType:winType
//  The type of window to create, see PortWindow.h for a list of
//  usable types.
//@descr
//  This constructor builds an PortWindow for a specified platformWindow and type.
//  The platformWindow can be created by a call to PortWindow::CreatePlatformWindow,
//  or can be generated in platform-dependent ways for specialized needs (for example,
//  if it needs a custom window class on Win32.)
//@endm
PortWindow::PortWindow(platformWindow theWin)
	{
	PortAssert(theWin != NULL);
	SubConstructWindow(theWin);
	}

void	PortWindow::SubConstructWindow(platformWindow theWin)
	{
	iWindow = theWin;
	iUserData = 0;
	iIsDialog = false;
	
	// Need to make a mapping between HWND and PortWindows somewhere.
	SetWindowLong(theWin, GWL_USERDATA, reinterpret_cast<long>(this));
	
	sWindowMap[theWin] = this;
	}

/*	
//@method:PortWindow::CountListItems
//@return:sInt16
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@descr
//  Returns the number of entries in the list box or menu.
//@endm
sInt16	PortWindow::CountListItems(LayoutItem theList)
	{
	sInt16 returnVal = 0;
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		returnVal = SendMessage(ctrl, LB_GETCOUNT, 0, 0);
	else if (iControlList[ind].iType == kAOControlPulldown)
		returnVal = SendMessage(ctrl, CB_GETCOUNT, 0, 0);
	return returnVal;
	}
	
//@method:PortWindow::GetListSelection
//@return:sInt16
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@descr
//  Returns the zero-based index to the selected item in the menu or list box.
//@endm
sInt16	PortWindow::GetListSelection(LayoutItem theList)
	{
	sInt16 returnVal = 0;
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		returnVal = SendMessage(ctrl, LB_GETCURSEL, 0, 0);
	else if (iControlList[ind].iType == kAOControlPulldown)
		returnVal = SendMessage(ctrl, CB_GETCURSEL, 0, 0);
	return returnVal;
	}
	
//@method:PortWindow::SelectListItem
//@return:sInt16
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@descr
//  Returns the zero-based index to the item to be selected in the menu or list box.
//@endm
void	PortWindow::SelectListItem(LayoutItem theList, sInt16 theIndex)
	{
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		SendMessage(ctrl, LB_SETCURSEL, theIndex, 0);
	else if (iControlList[ind].iType == kAOControlPulldown)
		SendMessage(ctrl, CB_SETCURSEL, theIndex, 0);
	}

	
//@method:PortWindow::GetTextOfListItem
//@return:PortString
//  The text of the specified list or menu item.
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@arg:theIndex:sInt16
//  The zero-based index to the item whose text you want to read.  Note that
//  this is not necessarily the selected item.  To get the selected item, call
//  GetListSelection() first to obtain the correct index.
//@descr
//  Returns the zero-based index to the selected item in the menu or list box.
//@endm
PortString	PortWindow::GetTextOfListItem(LayoutItem theList, sInt16 theIndex)
	{
	char buffer[256];
	PortString returnVal;
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		{
		SendMessage(ctrl, LB_GETTEXT, theIndex, (long)&buffer[0]);
		returnVal = buffer;
		}
	else if (iControlList[ind].iType == kAOControlPulldown)
		{
		SendMessage(ctrl, CB_GETLBTEXT, theIndex, (long)&buffer[0]);
		returnVal = buffer;
		}
	return returnVal;
	}
	
//@method:PortWindow::DeleteListItem
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@arg:theIndex:sInt16
//  The zero-based index to the item whose text you want to delete.  Note that
//  this is not necessarily the selected item.  To get the selected item, call
//  GetListSelection() first to obtain the correct index.
//@descr
//  Removes the specified item from the list or menu.
//
//  Note: After this call, the iListItems field in the AVControlList will no longer
//  be valid (it's not updated when the actual list changes).
//@endm
void	PortWindow::DeleteListItem(LayoutItem theList, sInt16 theIndex)
	{
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		{
		SendMessage(ctrl, LB_DELETESTRING, theIndex, 0);
		SendMessage(ctrl, LB_SETCURSEL, theIndex, 0);
		}
	else if (iControlList[ind].iType == kAOControlPulldown)
		{
		SendMessage(ctrl, CB_DELETESTRING, theIndex, 0);
		SendMessage(ctrl, CB_SETCURSEL, theIndex, 0);
		}
	}
	
//@method:PortWindow::AddListItem
//@arg:theItem:LayoutItem
//	A control id for a LISTBOX or PULLDOWN item.
//@arg:atPost:sInt16
//  The zero-based index to the item BEFORE which you want to place the
//  new entry (i.e. 0 is the front of the list, listsize+1 is the end).
//@arg:theItem:const PortString reference
//  The item to add
//@descr
//  Adds an item to the list or pulldown menu.
//
//  Note: After this call, the iListItems field in the AVControlList will no longer
//  be valid (it's not updated when the actual list changes).
//@endm
void	PortWindow::AddListItem(LayoutItem theList, sInt16 atPos, const PortString& theItem)
	{
	platformControl ctrl = GetLayoutControl(theList);
	long ind = GetControlListIndexFromCommand(theList);
	if (iControlList[ind].iType == kAOControlListBox)
		SendMessage(ctrl, LB_INSERTSTRING, atPos, (long)((const char*)theItem));
	else if (iControlList[ind].iType == kAOControlPulldown)
		SendMessage(ctrl, CB_INSERTSTRING, atPos, (long)((const char*)theItem));
	}

//@method:PortWindow::SetEditTextText
//@arg:theItem:LayoutItem
//	A control id for an edit text item.
//@arg:theText:PStringPtr
//	The text to set the Edit Text to.
//@descr
//  Sets the string displayed by an edit text item.  Can be called only while the
// dialog exists (i.e. during the DoModal() call).
//@endm
void	PortWindow::SetEditTextText(LayoutItem theItem, const PortString& theText)
	{
	platformControl eText = GetDlgItem(iWindow, theItem);
	SetWindowText(eText, (const char*)theText);
	}
	
//@method:PortWindow::GetEditTextText
//@arg:theItem:LayoutItem
//	A control id for an edit text item.
//@arg:theText:PStringPtr
//	The text currently in the edit text box.
//@descr
//  Returns the string displayed by an edit text item.  Can be called only while the
// dialog exists (i.e. during the DoModal() call).
//@endm
void	PortWindow::GetEditTextText(LayoutItem theItem,	PortString& theText)
	{
	platformControl eText = GetDlgItem(iWindow, theItem);
	sInt16 len = GetWindowTextLength(eText);
	char buffer[512];
	GetWindowText(eText, buffer, len+1);
	theText = buffer;
	}

//@method:PortWindow::SetStaticTextText
//@arg:theItem:LayoutItem
//	A control id for an static text item.
//@arg:theText:PStringPtr
//	The text to set the static text to.
//@descr
//  Sets the string displayed by an static text item.  Can be called only while the
// dialog exists (i.e. during the DoModal() call).  Note that this usually comes
// from the dialog template, so this is only called for text that changes during the
// dialog's execution.  Also note that the dialog will have been laid out before
// the first opportunity to call this (at InitialSetup()), so if the layout depends
// on the size of this text string, you'll need to change it in the control list
// before calling DoModal(), instead.
//@endm
void	PortWindow::SetStaticTextText(LayoutItem theItem, 	PStringPtr theText)
	{
	platformControl eText = GetDlgItem(iWindow, theItem);
	PStrToCStr(theText);
	SetWindowText(eText,  (char*)theText);
	CStrToPStr((char*)theText);
	}
*/

//@method:PortWindow::SetRadioButtonRange
//@arg:first:LayoutItem
//	The control id of the first button of the group.
//@arg:last:LayoutItem
//	The control id of the last button of the group.
//@arg:on:LayoutItem
//	The control id of the group button that should be "on"
//@descr
//	[deprecated] Sets all the buttons of a specified range to off, except for the item "on".  This
//  usually isn't necessary with controllist based dialogs, which use the group information
//  in the control list to figure this out.
//@endm
void	PortWindow::SetRadioButtonRange(LayoutItem first, LayoutItem last, LayoutItem on)
	{
	CheckRadioButton(iWindow, first, last, on);
	}
	
//@method:PortWindow::GetLayoutControl
//@return:platformControl
//	The control for the specified dialog item.
//@arg:theItem:LayoutItem
//	A control id for the item that you want the control for.
//@descr
// Get the control for the specified item.  Works for
// controls, radio buttons, checkboxes, and buttons.  Can be called only while the
// dialog exists (i.e. during the DoModal() call).
//@endm
platformControl	PortWindow::GetLayoutControl(LayoutItem theItem)
	{
	return GetDlgItem(iWindow, theItem);
	}

//@method:PortWindow::GetControlLayoutItem
//@return:LayoutItem
//  0 if there's no layout item associated with the control, otherwise the ID
//  of the layout item associated with the control.
//@arg:platformControl:theControl
//  The control to check
LayoutItem	PortWindow::GetControlLayoutItem(platformControl theControl)
	{
	LayoutItem id = 0;
	id = GetWindowLong(theControl, GWL_ID);
	return id;
	}
	
//@method:PortWindow::SelectLayoutItem
//@arg:theItem:LayoutItem
//	A control id the item to set focus to and select(only edit text does anything on the Mac).
//@descr
// Sets the focus to the specified item, which will only have an effect on the Mac
// if said item is edittext, in which case it will select it.  In Windows, sets the
// focus (the little dashed box, and the keyboard's attention) to that item.
// Can be called only while the dialog exists (i.e. during the DoModal() call).
//@endm
void	PortWindow::SelectLayoutItem(LayoutItem val)
	{
	platformControl eText = GetDlgItem(iWindow, val);
	SetFocus(eText);
	SendMessage(eText, EM_SETSEL, 0, 32768);
	}
	
	
//@method:PortWindow::SetFocusToLayoutItem
//@arg:theItem:LayoutItem
//	A control id the item to set keyboard focus to..
//@descr
// Sets the focus to the specified item, performing the default selection
// behavior for that item (if you want to "select all", use SelectLayoutItem
// instead.  Note that in the Mac, not all controls can have focus; this will do nothing
// if you attempt to focus to a control that can't accept it.  In Windows, sets the
// focus (the little dashed box, and the keyboard's attention) to that item.
// Can be called only while the dialog exists (i.e. during the DoModal() call).
//@endm
void	PortWindow::SetFocusToLayoutItem(LayoutItem val)
	{
	platformControl eText = GetDlgItem(iWindow, val);
	SetFocus(eText);
	}

//@method:PortWindow::EnableLayoutItem
//@arg:theItem:LayoutItem
//	A control id for the item to enable/disable
//@arg:enable:bool
//  If true, enables the item.  If false, disables it.
//@descr
//  Enables or disables the specified dialog control.
//@endm
void	PortWindow::EnableLayoutItem(LayoutItem val, bool enable)
	{
	platformControl theWindow = GetDlgItem(iWindow, val);
	EnableWindow(theWindow, enable);
	}
	

//@method:PortWindow::SetDefaultButton
//@arg:theItem:LayoutItem
//	A control id for the default button.
//@descr
// Sets the specified button to be the default, which has varying system effects.
// Usually it means that return/enter selects it, also sets up the "pulsing" on
// Aqua. Valid only for button controls.
//@endm
void	PortWindow::SetDefaultButton(LayoutItem val)
	{
	iDefault = val;
	SendMessage(iWindow, DM_SETDEFID, val, 0); 
	}

//@method:PortWindow::SetCancelButton
//@arg:theItem:LayoutItem
//	A control id for the cancel button.
//@descr
// Sets the specified button to be the "cancel" button (hit by esc, and by
// command-period on Mac).  Valid only for button controls.
//@endm
void	PortWindow::SetCancelButton(LayoutItem val)
	{
	iCancel = val;
	}
	
void	PortWindow::CreateActualControls(void)
	{
	long n = iControlList.size();

	// And set the values, default, and cancel buttons from the Control List
	if (n > 0)
		{
		for (long i = 0; i < n; i++)
			{
			platformControl ctrl = CreatePlatformControl(iControlList[i].iType, iWindow, 
					iControlList[i].iTitle, &iControlList[i].iRect, iControlList[i].iCommandID,
					iControlList[i].iNumericValue, iControlList[i].iDefault, 
					iControlList[i].iResourceRef, &iControlList[i].iTextValue);
					
			// Initialize the list-type ones
			switch (iControlList[i].iType)
				{
				case kAOControlListBox:
					for (long s = 0; s <iControlList[i].iListItems.size(); s++)
						SendMessage(ctrl, LB_ADDSTRING, 0, (long)((char*)iControlList[i].iListItems[s]));
					SendMessage(ctrl, LB_SETCURSEL, 0, 0);
					break;
				case kAOControlPulldown:
					for (long s = 0; s <iControlList[i].iListItems.size(); s++)
						SendMessage(ctrl, CB_ADDSTRING, 0, (long)((char*)iControlList[i].iListItems[s]));
					SendMessage(ctrl, CB_SETCURSEL, 0, 0);
					break;
				}
			SendMessage(ctrl, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT), TRUE);
			}
		}
		
	if (n > 0)
		{
		for (long i = 0; i < n; i++)
			{
			if (iControlList[i].iDefault)
				SetDefaultButton(iControlList[i].iCommandID);
			if (iControlList[i].iCancel)
				SetCancelButton(iControlList[i].iCommandID);
			}
		}

	}

	

//@method:PortWindow::PortWindow destructor
//@descr
//  Destructor for PortWindow.  It will delete the associated platform window,
//  and any OpenGL context.
//@endm
PortWindow::~PortWindow(void)
	{
	DestroyWindow(iWindow);
	sWindowMap.erase(iWindow);
	}


//@method:PortWindow::Invalidate
//@arg:rect:platformRect pointer
//@descr
//  Causes the specified rectangle of the window to be invalidated, which will
//  trigger an update event in the near future.
//@endm
void	PortWindow::Invalidate(platformRect* rect)
	{
	InvalidateRect(iWindow, rect, false);
	}
	
//@method:PortWindow::Validate
//@arg:rect:platformRect pointer
//@descr
//  Causes the specified rectangle of the window to be validated, removing
//  it from the area to be updated.  If this causes the invalid area to
//  be entirely removed, the pending update event will be suppressed.
//@endm
void	PortWindow::Validate(platformRect* rect)
	{
	ValidateRect(iWindow, rect);
	}
	
//@method:PortWindow::Update
//@arg:rect:platformRect pointer
//@descr
//  Called by the system when a window needs to be updated.  You will not usually
//  override this method; instead override Draw(), which it calls.
//@endm
void	PortWindow::Update(void)
	{
	PAINTSTRUCT ps;
	platformDC dc;
	dc.win = iWindow;
	dc.dc = BeginPaint(iWindow, &ps);
	// Do nothing, elegantly
	EndPaint(iWindow, &ps);
	}
	
//@method:PortWindow::ForceRedraw
//@descr
//  Causes the window to be redrawn immediately, outside of any event loop.
//  This is basically Invalidate() followed by Update(), but on OS X, windows
//  updated outside the event loop don't redraw because of double-buffering
//  issues, so this routine also flushes the QD buffer to fix that problem on X.
//@endm
void	PortWindow::ForceRedraw(void)
	{
	Invalidate();
	Update();
	}

	
//@method:PortWindow::Resize
//@arg:width:sInt16
//@arg:height:sInt16
//@descr
//	Resizes the window so that the *content area* has the specified width and height.
//	Note that this is different from Win32's general idea of window sizing, which sizes
//	based on the whole window, not the content area, but it matches the Mac way
//  of doing things.
//@endm
void	PortWindow::Resize(sInt16 h, sInt16 v)
	{
	RECT sizeRect;
	BOOL menu = (GetMenu(iWindow) != NULL) ? true : false;
	DWORD style = GetWindowLong(iWindow, GWL_STYLE);
	sizeRect.left = 0;
	sizeRect.right = h;
	sizeRect.top = 0;
	sizeRect.bottom = v;
	AdjustWindowRect(&sizeRect, style, menu);
	
	WINDOWPLACEMENT cPlace;
	cPlace.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(iWindow, &cPlace);
	MoveWindow(iWindow, cPlace.rcNormalPosition.left, cPlace.rcNormalPosition.top, 
				sizeRect.right - sizeRect.left, sizeRect.bottom - sizeRect.top, true);

	}

//@method:PortWindow::Move
//@arg:h:sInt16
//@arg:v:sInt16
//@descr
//	Moves the top of the client area to the specified position (h,v) in screen coordinates.
//@endm
void 	PortWindow::Move(int16 h, int16 v)
	{
	RECT cSize;
	GetClientRect(iWindow, &cSize);


	RECT sizeRect;
	BOOL menu = (GetMenu(iWindow) != NULL) ? true : false;
	DWORD style = GetWindowLong(iWindow, GWL_STYLE);
	sizeRect.left = 0;
	sizeRect.right = cSize.right-cSize.left;
	sizeRect.top = 0;
	sizeRect.bottom = cSize.bottom - cSize.top;
	AdjustWindowRect(&sizeRect, style, menu);
	
	WINDOWPLACEMENT cPlace;
	cPlace.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(iWindow, &cPlace);

	MoveWindow(iWindow, h, v, sizeRect.right-sizeRect.left, sizeRect.bottom-sizeRect.top, true);
	}


//@method:PortWindow::PostResize
//@arg:width:sInt16
//@arg:height:sInt16
//@descr
// Called after the window has been resized.  Override this in an PortWindow subclass if
// you need to know when windows are resized.  "width" and "height" are the new size of
// the window's content area.  If this is an OpenGL window, make sure to call the 
// inherited method to resize the associated OpenGL context.
//@endm
void	PortWindow::PostResize (sInt16 h, sInt16 v) 
	{
	// Eliminate a warning, but do nothing.
	h = h;
	v = v; 
	}
	

//@method:PortWindow::Hide
//@descr
// Hides the window (makes it invisible) without deleting it.
//@endm
void	PortWindow::Hide(void)
	{
	ShowWindow(iWindow, SW_HIDE);
	}
	
//@method:PortWindow::Show
//@descr
// Shows the window if it's hidden.  If it's not, has no effect.
//@endm
void	PortWindow::Show(void)
	{
	ShowWindow(iWindow, SW_SHOW);
	}
	
//@method:PortWindow::SetTitle
//@arg:title:PortString&
//@descr
// Sets or changes the title of the window to the specified title.
//@endm
void	PortWindow::SetTitle(const PortString&  title)
	{
	SetWindowText(iWindow, (const char*)title);
	}
	
//@method:PortWindow::OKToClose
//@return:bool
//  if "true" the window may close.  if "false", it cannot.
//@descr
//  Allows user confirmation on a window close.  If you return "true" from this 
//  method (the default if you don't override), the window will close.  If you
//  return false, the close will be "eaten."  This called in response to close box
//  clicks.
//@endm
	
//@method:PortWindow::Close
//@descr
// Called just before a window closes.  Override in an PortWindow subclass if you
// need to know when the window closes.  Note that by this point you cannot prevent
// the window from closing.  To do that, override OKToClose().
//@endm
void	PortWindow::Close(void)
	{
	// Override for specific behaviours, if needed.
	}
	
//@method:PortWindow::ClickIn
//@arg:x:sInt16
//@arg:y:sInt16
//@arg:flags:clickFlags
//@arg:capture:bool pointer
//@descr
//  (the old name for MouseDownIn, see that function)
//@endm

//static
//@method:PortWindow::GetFrontmostWindow
//@descr
// This static method returns the PortWindow that's frontmost in the
// application, or NULL if there isn't one.  Note that if a dialog is
// up, this function will return it.  Similarly, a floating palette will
// likely be the frontmost window.  If you need to limit to document windows
// only (a fairly common occurrance), you'll need to use the CPF-layer function
// CPFApp::GetFrontmostDocumentWindow();  
//@endm  
PortWindow*	PortWindow::GetFrontmostWindow(void)
	{
	HWND win = GetForegroundWindow();
	return(PortWindow::PlatformWindowToPortWindow(win));
	}

//@method:PortWindow::GetWindowContextSize
//@arg:x:sInt16 pointer
//@arg:y:sInt16 pointer
//@descr
//  Returns in *x and *y the size of the content area of the window.
//@endm
void	PortWindow::GetWindowContextSize(sInt16* x, sInt16* y)
	{
	RECT cSize;
	GetClientRect(iWindow, &cSize);
	*x = cSize.right;
	*y = cSize.bottom;
	}
	
//@method:PortWindow::GetWindowContextRect
//@arg:theRect:platformRect pointer
//@descr
//  Returns in *theRect the size of the content area of the window.
//@endm
void	PortWindow::GetWindowContextRect(platformRect* theRect)
	{
	RECT cSize;
	GetClientRect(iWindow, &cSize);
	SetRectTLBR(theRect, cSize.top, cSize.left, cSize.bottom, cSize.right);
	}
	
//@method:PortWindow::GetPlatformWindow
//@return:platformWindow
//  The platformWindow associated with this PortWindow.
//@descr
// It may occasionally be necessary to recover the platform-specific window
// type associated with the window.  This function returns it.
//@endm

//@method:PortWindow::GetViewController
//@return:AVViewController pointer
//  The view controller associated with this PortWindow.
//@descr
// This function returns the view controller associated with this window.  This
// only applies if the window is view based, otherwise it returns NULL.
//@endm

//@method:PortWindow::IsAcceleratedContext
//@return:bool
//  True if this OpenGl window is hardware accelerated.
//@descr
// For OpenGL windows, returns true if the context has hardware acceleration.  For
// non OpenGl windows, returns false.  Note that a context can lack acceleration for
// a variety of reasons, such as color depth or location, and that a false result from
// this doesn't necessarily mean that no accelerated context is possible on the
// specified machine.
//@endm

//@method:PortWindow::Is2DSizeIndependent
//@return:bool
//@descr
//  [OpenGL Only] Determines if OpenGL Windows in 2D mode use a size-
//  independent coordinate system (0-10, 0-10), or takes the OpenGL
//  coordinate system from the window's size.   Returns "true" by
//  default (uses the size-independent system), but your PortWindow subclass
//  can override this to return false, in which case the OpenGL coordinate
//  system will be dependent on the size of the window in 2D mode.
//@endm

//@method:PortWindow::SwapOpenGLBuffers
//@descr
// Exchanges the current and back buffers of the OpenGl context, causing the 
// buffered drawing to be displayed.   All OpenGL contexts created by PortableViews are
// double-buffered, so you'll always need to call this after drawing to an OpenGl
// window is completed.
//@endm

//@method:PortWindow::SetUserData
//@arg:userData:void pointer
//  The user data to associate with this window.
//@descr
// Stores the specified pointer (not what it points to) as a 4-byte value, without 
// interpretation.  The data itself is application defined.
//@endm

//@method:PortWindow::GetUserData
//@return:void pointer
//  The user data associated with this window.
//@descr
// Returns the user data for the window as a 4-byte value, without interpretation.  
// The data itself is application defined.
//@endm

//@method:PortWindow::ViewBasedWindow
//@return:bool
//  True if this window used the View controller model
//@descr
// Overload this to return "false" in an PortWindow subclass if you don't want to use the
// view-based model for windows (which, if you're making a subclass, you probably don't.)
//@endm

//@method:PortWindow::Draw
//@arg:theContext:AVDC pointer
//  A pointer to an AVDC attached to the context region of the window.
//@descr
//  Override this in an PortWindow subclass to draw the window.  View-based windows don't
//  override this; the draw commands are passed to the view.  OpenGL windows should
//  ignore the (probably NULL) context pointer passed in and just draw in OpenGL.
//@endm


} // namespace PortableViews;
#endif